//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//						(c) 1997 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	Extract.h --- abstract class for product creation
//
//	DESCRIPTION:
//		This is a class definition for an abstract object Extractor
//		which is capable of creating product files (images, spectra,
//		rates file) from event list.
//
//	CHANGE HISTORY:
//		UL, 20/02/98: 1.0.0: - first draft
//              DAH, 25/03/98: 1.0.0: - first real version
//              DAH, 06/04/98: 1.0.1: - make _copyGlobalAttribs method
//
//	AUTHOR:
//		D. Hinshaw
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al.,
//		[2]: Expanded description for SAS task `evselect'
//
//	$Header: /data/xmm12/cvsadmin/soc_repos/task_devel/packages/evselect/src/Extractor.h,v 1.29 2000/12/22 12:59:34 ulammers Exp $
//

#ifndef EXTRACTOR
#define EXTRACTOR

#include "Dal.h"
#include "errstr.h"

class Extractor {
private:

public:
  // empty default constructor
  Extractor(){}  
 
  // empty destructor
  ~Extractor(){} 
 
  // do the product extraction
  void doIt(Table *, bool, const string &, int, bool);
};


class Product {

protected:

  Table * _intable;
  Table * _tempTable;
  bool _destruct;
  const string _flagCol;
  int32 _flagBit;

  DataSet *_dataSet;
  real64 _binSize;
  vector<string> _globalAttribs;

  void _copyGlobalAttribs(Block *out);
  void getRange(const Column *col, real64 &min, real64 &max);
  void GetLimits(const Column *, string minparam, string maxparam, bool useparam,
		 vector<real64> &limits);
  Column * reType(const Column *oldCol, Dal::DataType newType);

public:

  // Product(){};

  Product(const string &name, Table *table, bool destructive,
	  const string &flagColumn, int flagBit);

  virtual void _getExposure(Block *productBlock);

  // destructor
  virtual ~Product();

};

class Image : public Product {

private:

public:

  Image(const string & name, Table *intable,
	bool destructive, const string &flagColumn, int flagBit) 
    : Product(name, intable, destructive, flagColumn, flagBit) {};

  void make(bool writeDss = false);

  void _getExposure(Block *productBlock);
}; 

class Spec : public Product {

public:

  Spec(const string & name, Table *intable,
	bool destructive, const string &flagColumn, int flagBit) 
    : Product(name, intable, destructive, flagColumn, flagBit) {};

  void make(bool writeDss = false);
}; 

class Rate : public Product {

private:

public:

  Rate(const string & name, Table *intable,
	bool destructive, const string &flagColumn, int flagBit) 
    : Product(name, intable, destructive, flagColumn, flagBit) {};
  void make(bool writeDss = false);
}; 

class Histo : public Product {

private:

public:

  Histo(const string & name, Table *intable,
	bool destructive, const string &flagColumn, int flagBit) 
    : Product(name, intable, destructive, flagColumn, flagBit) {};
  void make(bool writeDss = false);
}; 

#endif

